function c = loadnex(source_filename, eye_on_off, conversion)

%LOADNEX   reads a plexon file
%   Last modified: 8 July 2001

% Cortex encodes (check the Cortex file "encodes.h")
% we will move this to a small "included" file that people
% can modify according to their needs

TRIAL_START         = 150;
TRIAL_END           = 151;
ACTUAL_RESP         = [200 208]; 
EXPCTD_RESP         = [250 282];
REL_TRIAL_RESET     = 400;
BLOCKS              = [500 1000];
CONDITIONS          = [1000 32768];

verbose = 1;       %Display information during file translation
process_nops = 0;  %Flag to process NOPs (1:process, 0:don't)

FindX = inline('find(a>=b(1) & a<=b(2))','a','b');

fid = openfile(source_filename, 'r');
fname = Fnamext(source_filename);

%version number
c.version = 3;
%0: changes to object by the user not allowed; 1: changes allowed
c.changemode = 0;
%the name of the data file
c.name = fname;

%=====READ FILE INFO=====

[nvar, names, types, header_offsets, freq] = nexf_info(fid,verbose);

%=====READ STROBE/MARKER DATA (i.e. CORTEX ENCODES)=====

marker = find(types==6);
if(isempty(marker) | length(marker)>1)
    error('There is either no STROBED channel or too many markers channels in the file!');
end
if(~strcmp(deblank(names(marker,:)),'STROBED'))
    error('There is no "STROBED" marker variable in the file!');
end

[mtms, mval] = nexf_mark(fid, header_offsets(marker));

%convert timestamps to time in msec (values are rounded to nearest integer)
mtms = round((mtms/freq)*1000);

%process NOPs (set them to immediately preceding encode value) if flag set
if process_nops
    temp = find(mval==0);
    mval(temp) = mval(temp-1);
end

ntrials = length(find(mval==TRIAL_START));
if ~ntrials
    error('There are no TRIAL START markers in the datafile!!!');
else
    %total number of trials
    c.trials = ntrials;
end

%The freq of timestamps in the object.  Note that this is not the original
%freq in the datafile, but the result of the conversion to a msec timescale
%to keep consistent with Cortex conventions
c.KHz_resolution(1:ntrials,1) = 1;

known_codes = [];
%get conditions
temp = FindX(mval,CONDITIONS);
known_codes = [known_codes temp];
c.cond_no = mval(temp)' - CONDITIONS(1);
%get blocks
temp = FindX(mval,BLOCKS);
known_codes = [known_codes temp];
c.block_no = mval(temp)' - BLOCKS(1);
%get expected and actual responses (these match the cortex codes)
temp = FindX(mval,EXPCTD_RESP);
known_codes = [known_codes temp];
if ~isempty(temp)
    c.exp_response = mval(temp)' - EXPCTD_RESP(1);
else
    c.exp_response(1:ntrials,1) = -1; %just so that the expected response array is not empty
end
temp = FindX(mval,ACTUAL_RESP);
known_codes = [known_codes temp];
if ~isempty(temp)
    c.response = mval(temp)' - ACTUAL_RESP(1);
else
    c.response(1:ntrials,1) = 0; %just so that the response array is not empty
    disp('There were no Response encodes in the data file!');
end
%remove known codes from the general encode array
mval(known_codes) = [];
mtms(known_codes) = [];
%find the maximum number of encodes across trials
tsi = find(mval==TRIAL_START);
tei = find(mval==TRIAL_END);
max_codes = max(tei - tsi) + 1;
%fill encode arrays
c.codes = zeros(ntrials, max_codes);
c.times = zeros(ntrials, max_codes);
for ii = 1:ntrials
    c.codes(ii,1:(tei(ii)-tsi(ii)+1)) = mval(tsi(ii):tei(ii));
    c.times(ii,1:(tei(ii)-tsi(ii)+1)) = mtms(tsi(ii):tei(ii)) - mtms(tsi(ii));
end
%extract repeat, trial number, and trial type info
for ii=1:ntrials
    if(~isempty(find(c.codes(ii,:)==REL_TRIAL_RESET)))
        block_changept(ii) = 1;
    else
        block_changept(ii) = 0;
    end
end
numblocks = max(c.block_no);
c.repeat_no = cumsum(block_changept) + (numblocks-1);
c.repeat_no = floor(c.repeat_no/numblocks);
c.repeat_no = c.repeat_no';
trialcnt = 0;
for ii = 1:length(block_changept)
    if(block_changept(ii))
        trialcnt = 1;
    else
        trialcnt = trialcnt + 1;
    end
    c.rel_trial(ii) = trialcnt;
end
c.rel_trial = c.rel_trial';
c.trial_type = ones(ntrials, 1);

clear known_codes;
clear block_changept;

%=====READ SPIKE DATA=====

spkch = find(types==0);
nspks = length(spkch);
if(nspks) 
    c.cluster_bank = 1:nspks;
    c.spike_start = zeros (ntrials, nspks);
    c.spike_end = zeros (ntrials, nspks);
    %first calculate the total number of possible spikes and allocate spk array
    for ii = c.cluster_bank
        c.spike_channel(ii) = str2num(names(spkch(ii),4:6));
        fseek(fid, header_offsets(spkch(ii))+72, 'bof');
        spk_offset(ii) = fread(fid, 1, 'int32');
        nitems(ii) = fread(fid, 1, 'int32');
    end
    c.spikes = zeros(sum(nitems), 1);
    %now fill spike-related arrays
    total_spikes = 0;
    spkpos = 1;
    for ii = c.cluster_bank
        fseek(fid, spk_offset(ii), 'bof');
        tspk = fread(fid, [1 nitems(ii)], 'int32');
        %spike times in msec rounded to nearest integer
        tspk = round((tspk/freq)*1000);
        for jj = 1:length(tsi)
            temp = FindX(tspk,[mtms(tsi(jj)) mtms(tei(jj))]);
            if ~isempty(temp)
                c.spike_start(jj,ii) = spkpos;
                spkpos = spkpos + length(temp);
                c.spike_end(jj,ii) = spkpos-1;
                %spike times relative to trial start event
                c.spikes(c.spike_start(jj,ii) : c.spike_end(jj,ii)) = ...
                    tspk(temp) - mtms(tsi(jj));
                total_spikes = total_spikes + length(temp);
            end
        end            
    end
    c.spikes(total_spikes+1:end)=[];  %clear unfilled indices
    clear nitems;
    clear tspk;
    %make sure that the first spike of a trial is not at time zero.
    %this is necessary because of the way Giuseppe's spike analysis code operates.
    c.spikes(find(c.spikes==0))=1;
else
    c.cluster_bank  = [];
    c.spike_start   = [];
    c.spike_end     = [];
    c.spike_channel = [];
    c.spikes        = [];
end

%=====READ VERTICAL RETRACE SYNCH DATA=====

c.vretrace = zeros(ntrials,1);

vr = strmatch('EVENT002', names);
if isempty(vr)
    disp('This data file does not contain any vertical retrace synch data!');
else
    fseek(fid, header_offsets(vr)+72, 'bof');
    vr_offset = fread(fid, 1, 'int32');
    nitems = fread(fid, 1, 'int32');
    fseek(fid, vr_offset, 'bof');
    vraw = fread(fid, [1 nitems], 'int32');
    vraw = round((vraw/freq)*1000);
    for ii = 1:ntrials
        temp = vraw(FindX(vraw, [mtms(tsi(ii)) mtms(tei(ii))]));
        if(~isempty(temp))
            c.vretrace(ii, 1:length(temp)) = temp - mtms(tsi(ii));
        end
    end
    clear vraw;
end

%=====PREPARE TO READ CONTINUOUS DATA=====
c.eog_rate       = [];
c.eog2dva        = [];
c.eog_start      = zeros(ntrials, 1);
c.eog_ptr_start  = zeros(ntrials, 1);
c.eog_ptr_end    = zeros(ntrials, 1);
c.eog_X          = [];
c.eog_Y          = [];
c.lfp_start      = zeros(ntrials, 1);
c.lfp_ptr_start  = zeros(ntrials, 1);
c.lfp_ptr_end    = zeros(ntrials, 1);
c.lfp_channel    = [];
c.lfps           = [];

all_ad = strmatch('AD', names);
if isempty(all_ad)
    warning('This data file does not contain any A/D channels!');
    return;
else
    %Note: all A/D channels have the same number of points at the same time values.
    %Thus, these values need to be read/calculated once only
    [adfreq, ts, fn] = nexf_cont(fid, header_offsets(all_ad(1)));
    %this is the frequency of continuous (e.g., eog, lfp) data with respect
    %to timestamp data; it uses the same convention as in the Cortex menu
    sfreq = 1000/adfreq;
    c.eog_rate(1:ntrials, 1) = sfreq;
    ts = round((ts/freq)*1000);
end

%=====READ EYE POSITION DATA=====

%first 4 letters of eye position data channels
eogchnames = ['AD17';'AD18'];

if nargin > 2
    c.eog2dva = conversion;
    if isempty(eye_on_off)
        eogrange = [TRIAL_START TRIAL_END];
    else
        eogrange = eye_on_off;
    end        
    for ii = 1:size(eogchnames,1)
        varnum = strmatch(eogchnames(ii,:), names);
        if ~isempty(varnum)
            ad = nexf_cont(fid, header_offsets(varnum));
            if ii == 1
                eog_seekstart = mtms(find(mval==eogrange(1)));
                eog_seekend = mtms(find(mval==eogrange(2)));
                eogindx(1:ceil(sum(eog_seekend-eog_seekstart+(2*sfreq))/sfreq),1) = NaN;
                eogpos = 1;
                for jj = 1:ntrials
                    fr = 0;
                    for kk = 1:length(fn)
                        if(eog_seekstart(jj) >= ts(kk) & eog_seekstart(jj) <= ts(kk)+(fn(kk)-1)*sfreq)
                            fr = kk;
                            break;
                        end
                    end
                    if (~fr)
                        warning(['Did not find eye movement points in trial' num2str(jj)]);
                    end
                    s_ind = round (((eog_seekstart(jj)-ts(fr))/sfreq) + 1 + sum(fn(1:fr-1)));
                    e_ind = ceil (((eog_seekend(jj)-ts(fr))/sfreq) + 1 + sum(fn(1:fr-1)));
                    %c.eog_start(jj,1) = ts(fr) + (s_ind-1-sum(fn(1:fr-1)))*sfreq;
                    c.eog_start(jj,1) = c.times(jj, find(c.codes(jj,:)==eogrange(1)));
                    c.eog_ptr_start(jj,1) = eogpos;
                    eogpos = eogpos + (e_ind - s_ind) + 1;
                    c.eog_ptr_end(jj,1) = eogpos-1;
                    eogindx(c.eog_ptr_start(jj,1) : c.eog_ptr_end(jj,1)) = [s_ind:e_ind];
                end
                eogindx(find(isnan(eogindx))) = [];
                c.eog_X = ad(eogindx)';
            else
                c.eog_Y = ad(eogindx)';
            end                      
        end
    end
    clear eog_seekstart;
    clear eog_seekend;
    clear eogindx;
    clear ad;
end

numeogs = 0;
for ii = 1:2
    if ~isempty(strmatch(eogchnames(ii,:), names))
        numeogs = numeogs + 1;
    end
end
if(numeogs & nargin==2)
    disp('The data file contained EyePos data which was not processed...');
    disp('...see input arguments to the function.');
end    

%=====READ LOCAL FIELD POTENTIAL (LFP) DATA=====

%first 4 letters of LFP data channels
lfpchnames = ['AD01';'AD02';'AD03';'AD04';'AD05';'AD06';'AD07';'AD08';...
              'AD09';'AD10';'AD11';'AD12';'AD13';'AD14';'AD15';'AD16'];
%LFPs are processed from trial start to trial end
lfprange   = [TRIAL_START TRIAL_END];
lfpnum     = 0;

%check to see if there are any LFP data to process
totlfp = length(all_ad) - numeogs;    

if totlfp > 0
    for ii = 1:size(lfpchnames,1)
        varnum = strmatch(lfpchnames(ii,:), names);
        if ~isempty(varnum)
            lfpnum = lfpnum + 1;
            c.lfp_channel(lfpnum) = str2num(names(varnum,3:4));
            ad = nexf_cont(fid, header_offsets(varnum));
            if lfpnum == 1
                lfp_seekstart = mtms(find(mval==lfprange(1)));
                lfp_seekend = mtms(find(mval==lfprange(2)));
                lfpindx(1:ceil(sum(lfp_seekend-lfp_seekstart+(2*sfreq))/sfreq),1) = NaN;
                lfppos = 1;
                for jj = 1:ntrials
                    fr = 0;
                    for kk = 1:length(fn)
                        if(lfp_seekstart(jj) >= ts(kk) & lfp_seekstart(jj) <= ts(kk)+(fn(kk)-1)*sfreq)
                            fr = kk;
                            break;
                        end
                    end
                    if (~fr)
                        warning(['Did not find lfp points in trial' num2str(jj)]);
                    end
                    s_ind = floor (((lfp_seekstart(jj)-ts(fr))/sfreq) + 1 + sum(fn(1:fr-1)));
                    e_ind = ceil (((lfp_seekend(jj)-ts(fr))/sfreq) + 1 + sum(fn(1:fr-1)));
                    %c.lfp_start(jj,1) = ts(fr) + (s_ind-1-sum(fn(1:fr-1)))*sfreq;
                    c.lfp_start(jj,1) = c.times(jj, find(c.codes(jj,:)==lfprange(1)));
                    c.lfp_ptr_start(jj,1) = lfppos;
                    lfppos = lfppos + (e_ind - s_ind) + 1;
                    c.lfp_ptr_end(jj,1) = lfppos-1;
                    lfpindx(c.lfp_ptr_start(jj,1) : c.lfp_ptr_end(jj,1)) = [s_ind:e_ind];
                end
                lfpindx(find(isnan(lfpindx))) = [];
                c.lfps = zeros(length(lfpindx), totlfp);
            end                      
            c.lfps(1:length(lfpindx), lfpnum) = ad(lfpindx);
        end
    end
    clear lfp_seekstart;
    clear lfp_seekend;
    clear lfpindx;
    clear ad;
end

fclose(fid);

s = sprintf('\nData file processed.\n');
disp(s);

return % to main
