function [X,Y,D,U] = trialtouch (c, trial, cue, m)

%@CORTEX/TRIALTOUCH  returns touchscreen positions from a @cortex trial
%
%   [POSX,POSY,DOWN,UP] = TRIALTOUCH(COBJ,TRIAL,CUE,MODE) retrieves XY
%   position and timing of touchscreen data from a single TRIAL in the
%   COBJ @cortex object. The optional string argument MODE determines
%   how events are analyzed. CORTEX samples the touchscreen state at
%   the rate specified in cortex.cfg, and data are collected any time
%   a contact is detected. Thus, since a typical "touch response" may
%   last longer than the sampling interval, multiple events may be
%   recorded for each discrete motor response. Valid MODEs are:
%
%   'all':   all samples are included in the output (this is the
%            default, used if MODE is omitted)
%   'down':  only the XY position of the first contact with the
%            screen is returned
%   'up':    only the XY position just prior to the release of a
%            contact are returned
%   'mean':  XY positions are averaged over the entire duration of
%            the contact
%
%   POSX and POSY are double arrays of X and Y touchscreen
%   coordinates; DOWN and UP are double arrays with the timing in ms
%   of the initial contact and of the release of each touch response,
%   relative to the event specified by the scalar CUE; if MODE=='all'
%   then both DOWN and UP contain the timing of each touchscreen
%   sample.
%
%   [...] = TRIALTOUCH(COBJ,TRIAL,MODE)  same as above, except that the
%   timing of touchscreen events is computed relative to the beginning
%   of the trial.
%
% See also: @CORTEX/TOUCH
%
% Last modified: 28 Oct 04

% Created on 11 Oct 04
% 28 Oct 04: cosmetics

if isempty (c.tch), error ('no touchscreen data in this file'); end
mode = 0;
switch nargin
   case 2
      % (COBJ,TRIAL)
      cue = [];
   case 3
      % (COBJ,TRIAL,CUE)
      if ~isnumscalar(cue)
         % (COBJ,TRIAL,MODE)
         mode = whatmode(cue);
         cue = [];
      end
   case 4
      % (COBJ,TRIAL,CUE,MODE)
      if ~isnumscalar(cue)
         error ('CUE argument should be a scalar');
      end
      mode = whatmode(m);
   otherwise
      error ('wrong number of arguments');
end
[X,Y,D,U] = gettouch (c,trial,cue,mode);
if any(isnan(D)), warning ('cue event not found in requested trial'); end