function [dense, uuni, ugrp] = meanfiringdensity (sp, win, varargin)

%@SPIX/MEANFIRINGDENSITY  spike-density vectors from a @spix object
%
%   DE = DENSITY(SP,EPOCH) returns spike-density vectors computed from the SP
%   @spix object, over the epoch specified in EPOCH, a two-element array with
%   the start and end ms relative to the cue event.
%
%   EPOCH can also be one of the following strings:
%   'all': selects the entire time window contained in SP
%   'pre': selects the epoch preceding the cue event 
%   'pos': selects the epoch following the cue event
%   'var': instead of time-locking to the cue event, previously
%          defined, cell-by-cell latencies are used to align trials
%
%   The smoothing kernel's shape and width can be changed by setting the
%   two preference values density_kernel_type and density_kernel_width. See
%   >> help setpreference
%   Currently available kernel types are 'gauss', 'boxcar', and 'epsp'.
%
%   DE is a 3-D matrix of density values; each row represents one processed
%   unit; each column corresponds to one ms; density plots for different
%   keys or key groupings are arranged along the 3rd dimension.
%
%   If no other arguments are specified, then all currently included units
%   are processed for each of the currently included keys.
%
%   DE = MEANFIRINGDENSITY(..., restrictions) returns only trial epochs
%   specified by RESTRICTIONS. See online help for RESTRICTIONS usage.
%
%   [SPMAT,UNITS,CRIT] = MEANFIRINGDENSITY(...) also returns an array of
%   UNITS from which each trial epoch derives, and CRIT, an array of keys,
%   combos, or levels matching each trial.
%
%Last modified: 18 Feb 06

% Since 23 Aug 03:
% 18 Feb 06: smooth replaces smoothfr

[spmat, uni, grp] = spikematrix (sp, win, bin, varargin{:});
uuni = unique(uni); nuni = length(uuni);
ugrp = unique(grp); ngrp = length(ugrp);
dense = zeros (nuni, size(spmat,2), ngrp);
for cc = 1:length(uuni)
   for gr = 1:length(ugrp)
      chunk = spmat (uni==uuni(cc) & grp==ugrp(gr), :);
      te = smooth (chunk, kwidth, ktype);
      dense(cc,:,gr) = te;
   end
end
