function as = addata (as, data, levels)

%@ANNSETUP/ADDATA   adds observations to an @annsetup object
%
%   AS = ADDATA(AS,DATA,LEVELS) adds to AS the set of observations
%   contained in the numeric vector DATA; LEVELS specifies the levels
%   associated to the data (one level per factor, obviously). LEVELS can be
%   a numeric vector, a cell array, or a character matrix (one row per
%   level).
%
%   For example:
%   >> as=annsetup(3); % object creation for a 3-way anova
%   >> d=[6 5 7 6 8 7 5 7 6]; % the data associated with...
%   >> g={'blue',1,'up'};     % ...the three factor levels
%   >> as=addata(as,d,g);
%   >> ...
%
%   Note that the object variable must appear both as the first input
%   argument and as the output.
%
%Last modified: 2 Aug 03

% handle data
if ~isnumeric(data) | prod(size(data)) ~= length(data)
   error ('argument ''data'' must be a numeric vector');
end
if size(data, 2) > 1, data = data'; end
as.D = [as.D; data];

% handle levels
if ischar (levels), levels = cellstr(levels); end
if length (levels) ~= as.N
   error  (['wrong number of levels: this is a ' num2str(as.N) '-factor @annsetup object']);
end
for ii = 1:as.N
   lev = levels(ii);
   if iscell(lev)
      lev = lev{1};
   elseif ~isnumeric(lev)
      error ('invalid level type (use either numeric or cell)');
   end
   if ischar(lev), lev = {lev}; end
   if ~isempty(as.G{ii}) & ~strcmp (class(as.G{ii}), class(lev))
      error ('level type mismatch');
   end
   rr = repmat (lev, length(data), 1);
   tt = as.G{ii};
   ot = [tt; rr];
   as.G {ii} = ot;
end
if as.verbose
    info (as)
end
